<?php

namespace Src\View;

use Src\App\WordpressAdapterInterface;
use Src\Infrastructure\FloraLMSAdapter;

class SettingsView
{
    const ADMIN_PAGE_SLUG = 'flora_lms';
    const ADMIN_PAGE_TITLE = 'FloraLMS Plugin';
    const ADMIN_MENU_TITLE = 'FloraLMS';

    const AUTH_HELP_TEXT = 'Copy your secret key from SSO-JWT settings your Flora account';
    const DOMAIN_HELP_TEXT = 'Copy your domain name of your Flora account';
    const IMPORT_COURSES = 'Manually import courses from Flora LMS';
    const ADD_GO_TO_LINK = 'You can add this link to site to go to Flora LMS portal';

    const SSO_JWT_SETTINGS = 'SSO-JWT settings';
    const IMPORT_FROM_FLORA = 'Import courses from Flora LMS';
    const SECURITY_KEY = 'Security Key';
    const DOMAIN = 'Domain';
    const LINK_TO_FLORA = 'Test Flora LMS login link';
    const CLICK_TO_IMPORT = 'Click to import';
    const INFO_TEXT = 'About this link';
    const TEXT_FOR_AUTH_LINK = 'Go to Flora LMS';

    private $isRegistrationDataProvided;
    private $updateCallback;
    private $saveSecretKeyCallback;
    private $saveDomainCallback;
    private $wordpressAdapter;

    public function __construct($isRegistrationDataProvided, $updateCallback, $saveSecretKeyCallback, $saveDomainCallback, WordpressAdapterInterface $wordpressAdapter)
    {
        $this->isRegistrationDataProvided = $isRegistrationDataProvided;
        $this->updateCallback = $updateCallback;
        $this->saveSecretKeyCallback = $saveSecretKeyCallback;
        $this->saveDomainCallback = $saveDomainCallback;
        $this->wordpressAdapter = $wordpressAdapter;
    }

    public function register()
    {
        /** @see SettingsView::registerAdminPage() **/
        $this->wordpressAdapter->addAdminMenuAction(array($this, 'registerAdminPage'));
        /** @see SettingsView::registerCustomAdminFields() **/
        $this->wordpressAdapter->addAdminMenuAction(array($this, 'registerCustomAdminFields'));
        /** @see SettingsView::settingsLink() **/
        $this->wordpressAdapter->registerSettingsLinks(array($this, 'settingsLink'));
    }

    public function registerAdminPage()
    {
        $this->wordpressAdapter->addAdminMenuPage($this->getSettingsPage());
    }

    public function registerCustomAdminFields()
    {
        $this->wordpressAdapter->registerAdminFields($this->getSettings(), $this->getSections(), $this->getFields());
    }

    public function settingsLink($links)
    {
        $link = self::ADMIN_PAGE_SLUG;
        $settingLink = "<a href='admin.php?page=$link'>Settings</a>";
        $links[] = $settingLink;
        return $links;
    }

    public function authenticationInput()
    {
        $value = $this->wordpressAdapter->getFieldValueById('secret_key');
        echo $this->questionMark(self::AUTH_HELP_TEXT) . '<input type="text" class="regular-text" name="secret_key" maxlength=255 id="secret_key" value="' . $value . '" />';
    }

    public function domainInput()
    {
        $value = $this->wordpressAdapter->getFieldValueById('domain');
        echo $this->questionMark(self::DOMAIN_HELP_TEXT) . '<input type="text" class="regular-text" name="domain" maxlength=50 id="domain" value="' . $value . '" />';
    }

    public function updateSectionCallback()
    {
        echo '<span>' . self::IMPORT_COURSES . '</span>';
    }

    public function authLink()
    {
        if (!$this->isRegistrationDataProvided)
        {
            echo '<span>No data provided for registration</span>';
            return;
        }
        $url = FloraLMSAdapter::addGoToFloraLms($this->wordpressAdapter);
        $text = self::TEXT_FOR_AUTH_LINK;
        $title = strip_tags($text);
        echo "<a title='{$title}' href='{$url}' class='go-to-flora-link'>{$text}</a>";
    }

    public function updateButton()
    {
        echo '<input type="submit" name="submit" id="submit" class="button button-primary" value="Update courses">';
    }

    public function noticeText()
    {
        echo self::ADD_GO_TO_LINK;
    }


    public function questionMark($help_text)
    {
        return "<div style='cursor:default;float:left;height:32px; width:20px; display:flex; align-items:center;' title='$help_text'><div style='background-color: #23282d;
        border-radius: 50%;
        color: white;
        height: 18px;
        display: flex;
        width: 18px;
        justify-content: center;
        align-items: center;'>?</div></div>";
    }

    public function getSettingsPage()
    {
        return array(
            'page_title' => self::ADMIN_PAGE_TITLE,
            'menu_title' => self::ADMIN_MENU_TITLE,
            'capability' => 'manage_options',
            'menu_slug' => self::ADMIN_PAGE_SLUG,
            'callback' => function () {
                require_once($this->wordpressAdapter->retrievePluginDirPath() . '/templates/admin.php');
            },
            'icon_url' => $this->wordpressAdapter->concatenateWithPluginUrl('assets/flora.jpg'),
            'position' => 60
        );
    }

    public function getSettings()
    {
        return array(
            array(
                'option_group' => 'plugin_option_group',
                'option_name' => 'secret_key',
                'callback' => $this->saveSecretKeyCallback
            ),
            array(
                'option_group' => 'plugin_option_group',
                'option_name' => 'domain',
                'callback' => $this->saveDomainCallback
            ),
            array(
                'option_group' => 'plugin_update_group',
                'option_name' => 'update_products_list',
                'callback' => $this->updateCallback
            )
        );
    }

    public function getSections()
    {
        return array(
            array(
                'id' => 'plugin_admin_index',
                'title' => self::SSO_JWT_SETTINGS,
                'callback' => array(),
                'page' => 'sso_jwt_settings'
            ),
            array(
                'id' => 'plugin_admin_update_products',
                'title' => self::IMPORT_FROM_FLORA,
                /** @see SettingsView::updateSectionCallback() **/
                'callback' => array($this, 'updateSectionCallback'),
                'page' => 'update_section'
            )
        );
    }

    public function getFields()
    {
        return array(
            array(
                'id' => 'secret_key',
                'title' => self::SECURITY_KEY,
                /** @see SettingsView::authenticationInput() **/
                'callback' => array($this, 'authenticationInput'),
                'page' => 'sso_jwt_settings',
                'section' => 'plugin_admin_index',
                'args' => array(
                    'label_for' => 'secret_key',
                )
            ),
            array(
                'id' => 'domain',
                'title' => self::DOMAIN,
                /** @see SettingsView::domainInput() **/
                'callback' => array($this, 'domainInput'),
                'page' => 'sso_jwt_settings',
                'section' => 'plugin_admin_index',
                'args' => array(
                    'label_for' => 'domain',
                )
            ),
            array(
                'id' => 'update_button',
                'title' => self::CLICK_TO_IMPORT,
                /** @see SettingsView::updateButton() **/
                'callback' => array($this, 'updateButton'),
                'page' => 'update_section',
                'section' => 'plugin_admin_update_products',
                'args' => array()
            ),
            array(
                'id' => 'auth_link',
                'title' => self::LINK_TO_FLORA,
                /** @see SettingsView::authLink() **/
                'callback' => array($this, 'authLink'),
                'page' => 'update_section',
                'section' => 'plugin_admin_update_products',
                'args' => array()
            ),
            array(
                'id' => 'notice_text',
                'title' => self::INFO_TEXT,
                /** @see SettingsView::noticeText() **/
                'callback' => array($this, 'noticeText'),
                'page' => 'update_section',
                'section' => 'plugin_admin_update_products',
                'args' => array()
            )
        );
    }
}