<?php

namespace Src\Infrastructure;

use Src\App\FloraLMSAdapterInterface;
use Src\App\HttpClientInterface;
use Src\App\WordpressAdapterInterface;
use Src\App\RepositoryInterface;

use Firebase\JWT\JWT;

class FloraLMSAdapter implements  FloraLMSAdapterInterface
{
    const REST_NAMESPACE = 'flora-lms/v1';
    const REST_REF_PARAM = 'ref';
    const REST_CONTENT_LINK_PARAM = '(?P<ref>[a-zA-Z0-9-:/.]+)';

    const REST_GO_TO_FLORA = '/go-to-flora';
    const REDIRECT_URL = '&redirect_url=';
    const EMAIL_PARAM = 'email';

    const DATE_STANDARD = 'Y-m-d H:i:s';

    const REQUEST_PROTOCOL = 'https://';
    const REQUEST_POSTFIX = '/api.php/api/v1/marketplace/list?timestamp=';
    const FLORA_HTTP_POSTFIX = '?no_jwt=1';

    const FLORA_JWT_LOGIN_URL = '/sso/login/jwt?jwt=';
    const FLORA_JWT_ENCODE_ALG = 'HS256';
    const TOKEN_LENGTH = 32;
    const EXPIRATION_TIME = 30;

    /**
     * @param string $domainName
     * @param int $timestamp
     * @param HttpClientInterface $httpRequest
     * @return array
     */
    public function retrieveCoursesList($domainName, $timestamp, HttpClientInterface $httpRequest)
    {
        if ($timestamp < 0)
        {
            $timestamp = 0;
        }
        $url = self::REQUEST_PROTOCOL . $domainName . self::REQUEST_POSTFIX . $timestamp;
        $response = $httpRequest->httpRequest($url, array(
            'sslverify' => false,
        ));

        if ($response['response']['code'] !== 200)
        {
            throw new \RuntimeException('Unable to fetch courses');
        }
        return $httpRequest->retrieveAndParseBody($response);
    }

    /**
     * @param string $userEmail
     * @param array $authData
     * @return string|null
     * @throws \Exception
     */
    public function generateJWTLink($userEmail, $authData)
    {
        if ($authData['secret_key'] === '' || $authData['domain_name'] === '')
        {
            return null;
        }

        $jwt = $this->generateJwtToken($authData['secret_key'], $userEmail);

        $domainName = $authData['domain_name'];
        return self::REQUEST_PROTOCOL . $domainName . self::FLORA_JWT_LOGIN_URL . $jwt;
    }

    /**
     * @param string $restUrl
     * @return string
     */
    public function getActionUrl($restUrl)
    {
        return $restUrl . self::REST_NAMESPACE . '/' . self::REST_REF_PARAM . '=';
    }

    /**
     * @param string $secretKey
     * @param string $userEmail
     * @return string|null
     * @throws \Exception
     */
    private function generateJwtToken($secretKey, $userEmail)
    {
        if (!$userEmail)
        {
            return null;
        }
        $tokenId = base64_encode(random_bytes(self::TOKEN_LENGTH));
        $issuedAt = time();
        $expire = $issuedAt + self::EXPIRATION_TIME;
        $data = array(
            'iat' => $issuedAt,
            'jti' => $tokenId,
            'exp' => $expire,
            'email' => $userEmail
        );

        return JWT::encode(
            $data,
            $secretKey,
            self::FLORA_JWT_ENCODE_ALG
        );
    }

    /**
     * @param WordpressAdapterInterface $wordpressAdapter
     * @return string
     */
    public static function addGoToFloraLMS(WordpressAdapterInterface $wordpressAdapter)
    {
        return $wordpressAdapter->getRestUrl() . self::REST_NAMESPACE . self::REST_GO_TO_FLORA;
    }
}