<?php

namespace Src\Infrastructure;

use Src\App\WordpressAdapterInterface;
use Src\View\CoursesView;

class WordpressAdapter implements WordpressAdapterInterface
{
    const PLUGIN_FILE = 'flora-lms.php';

    /**
     * @param $callable
     */
    public function registerActivationCallback($callable)
    {
        register_activation_hook($this->getPluginFilePath(), $callable);
    }

    /**
     * @param array $callable
     */
    public function registerDeactivationCallback($callable)
    {
        register_deactivation_hook($this->getPluginFilePath(), $callable);
    }

    /**
     * @param array $callable
     */
    public function addActivatedPluginAction($callable)
    {
        add_action('activated_plugin', $callable);
    }

    /**
     *
     */
    public function flushRewriteRules()
    {
        flush_rewrite_rules();
    }

    /**
     * @param $postType
     */
    public function unregisterPostType($postType)
    {
        unregister_post_type($postType);
    }

    /**
     * @param $id
     * @return string
     */
    public function getFieldValueById($id)
    {
        return esc_attr(get_option($id));
    }

    /**
     * @param array $callable
     */
    public function addAdminMenuAction($callable)
    {
        add_action('admin_menu', $callable);
    }

    /**
     * @param array $page
     */
    public function addAdminMenuPage(array $page)
    {
        add_menu_page(
            $page['page_title'],
            $page['menu_title'],
            $page['capability'],
            $page['menu_slug'],
            $page['callback'],
            $page['icon_url'],
            $page['position']
        );
    }

    /**
     * @param array $settings
     * @param array $sections
     * @param array $fields
     */
    public function registerAdminFields(array $settings, array $sections, array $fields)
    {
        foreach ($settings as $setting) {
            register_setting($setting['option_group'], $setting['option_name'], isset($setting['callback']) ? $setting['callback'] : '');
        }

        foreach ($sections as $section) {
            add_settings_section($section['id'], $section['title'], isset($section['callback']) ? $section['callback'] : '', $section['page']);
        }

        foreach ($fields as $field) {
            add_settings_field($field['id'], $field['title'], isset($field['callback']) ? $field['callback'] : '', $field['page'], $field['section'], isset($field['args']) ? $field['args'] : '');
        }
    }

    /**
     * @param $url
     * @param array $args
     * @return array
     */
    public function httpRequest($url, $args = [])
    {
        return wp_remote_get($url, $args);
    }

    /**
     * @param $response
     * @return array|mixed
     */
    public function retrieveAndParseBody($response)
    {
        return json_decode(wp_remote_retrieve_body($response), true);
    }

    /**
     * @param array $callable
     */
    public function addActionOnInitialization($callable)
    {
        add_action('init', $callable);
    }

    /**
     * @param array $callable
     */
    public function addFilterOnDeletePost($callable)
    {
        add_filter('pre_delete_post', $callable, 10, 3);
    }

    /**
     * @param array $callable
     */
    public function addFilterOnTrashPost($callable)
    {
        add_filter('pre_trash_post', $callable, 10, 2);
    }

    /**
     * @param $postTypeName
     */
    public function registerCustomPostType($postTypeName)
    {
        register_post_type(
            $postTypeName,
            array(
                'labels' => array(
                    'name' => "{$postTypeName}s",
                    'singular_name' => "$postTypeName single",
                    'add_new' => 'Add New',
                    'add_new_item' => "Add new $postTypeName",
                    'edit' => 'Edit',
                    'edit_item' => "Edit $postTypeName",
                    'new_item' => "New $postTypeName",
                    'view' => 'View',
                    'view_item' => "View $postTypeName",
                    'search_items' => "Search $postTypeName`s",
                    'not_found' => "No $postTypeName found",
                    'not_found_in_trash' => "No $postTypeName found in Trash",
                    'parent' => "Parent $postTypeName"
                ),
                'public' => true,
                'menu_position' => 15,
                'supports' => array('title', 'editor', 'thumbnail'),
                'has_archive' => true,
                'menu_icon' => plugins_url('assets/flora.jpg', plugin_basename(dirname(__FILE__, 3)) . '/' . self::PLUGIN_FILE)
            )
        );
    }

    /**
     * @param array $post
     * @return string
     */
    public function insertPost(array $post)
    {
        return wp_insert_post($post, '');
    }

    /**
     * @return string
     */
    public function getRestUrl()
    {
        return get_rest_url();
    }

    /**
     * @return \Src\App\WP_User
     */
    public function getCurrentUser()
    {
        return wp_get_current_user();
    }

    /**
     * @param string $namespace
     * @param string $restParam
     * @param array $args
     */
    public function registerAction($namespace, $restParam, $args)
    {
        add_action('rest_api_init', function () use ($namespace, $restParam, $args) {
            register_rest_route($namespace, $restParam, $args);
        });
    }

    /**
     * @param array $callable
     */
    public function registerSettingsLinks($callable)
    {
        add_filter('plugin_action_links_' . plugin_basename(dirname(__FILE__, 3)) . '/' . self::PLUGIN_FILE, $callable);
    }

    /**
     * @param array $callable
     */
    public function addActionToTemplates($callable)
    {
        add_filter('template_include', $callable);
    }

    /**
     * @param array $page
     */
    private function updateCoursesPage($page)
    {
        wp_update_post($page);
    }

    /**
     * @param string $pageName
     * @return string
     */
    private function findPageByTitle($pageName)
    {
        return get_page_by_title($pageName);
    }

    /**
     * @param string $pageName
     * @return void
     */
    public function insertCoursesPage($pageName)
    {
        $coursesPage = $this->findPageByTitle($pageName);

        if (!$coursesPage)
        {
            $coursesPage = array(
                'post_title' => $pageName,
                'post_content' => '',
                'post_status' => 'publish',
                'post_author' => $this->getCurrentUser()->ID,
                'post_type' => 'page',
            );
            $this->insertPost($coursesPage);
        } else {
            $coursesPage->post_status = 'publish';
            $this->updateCoursesPage($coursesPage);
        }
    }

    /**
     * @return string
     */
    public function getPluginFilePath()
    {
        return plugin_basename(dirname(__FILE__, 3)) . '/' . self::PLUGIN_FILE;
    }

    /**
     * @param string $path
     * @return string
     */
    public function concatenateWithPluginUrl($path)
    {
        return plugins_url($path, $this->getPluginFilePath());
    }


    /**
     * @return string
     */
    public function retrievePluginDirPath()
    {
        return plugin_dir_path(dirname(__FILE__, 2));
    }

    public function getCourseContentHtml(\Src\App\Course $course, $actionUrl)
    {
       return CoursesView::getContent($course, $actionUrl);
    }
}